"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateShrinkwrapTask = void 0;
const GulpTask_1 = require("./GulpTask");
const path = require("path");
const semver = require("semver");
const node_core_library_1 = require("@rushstack/node-core-library");
/**
 * This task attempts to detect if package.json file has been updated without the
 * shrinkwrap file being regenerated.
 *
 * It does this by checking that every dependency and dev dependency exists in the
 * shrinkwrap file and that the version in the shrinkwrap file satisfies what is
 * defined in the package.json file.
 * @public
 */
class ValidateShrinkwrapTask extends GulpTask_1.GulpTask {
    /**
     * Instantiates an instance of the ValidateShrinkwrap task
     */
    constructor() {
        super('validate-shrinkwrap');
    }
    /**
     * Iterates through dependencies listed in a project's package.json and ensures that they are all
     * resolvable in the npm-shrinkwrap file.
     */
    executeTask(gulp, completeCallback) {
        const pathToPackageJson = path.join(this.buildConfig.rootPath, node_core_library_1.FileConstants.PackageJson);
        const pathToShrinkwrap = path.join(this.buildConfig.rootPath, 'npm-shrinkwrap.json');
        if (!this.fileExists(pathToPackageJson)) {
            this.logError('Failed to find package.json at ' + pathToPackageJson);
            return;
        }
        else if (!this.fileExists(pathToShrinkwrap)) {
            this.logError('Failed to find package.json at ' + pathToShrinkwrap);
            return;
        }
        // eslint-disable-next-line
        const packageJson = require(pathToPackageJson);
        // eslint-disable-next-line
        const shrinkwrapJson = require(pathToShrinkwrap);
        this._validate(packageJson.dependencies, shrinkwrapJson.dependencies);
        this._validate(packageJson.devDependencies, shrinkwrapJson.dependencies);
        return;
    }
    _validate(packageDep, shrinkwrapDep) {
        for (const pkg in packageDep) {
            if (!shrinkwrapDep.hasOwnProperty(pkg)) {
                this.logError(`Failed to find package ${pkg} in shrinkwrap file`);
            }
            else if (!semver.satisfies(shrinkwrapDep[pkg].version, packageDep[pkg])) {
                this.logError(`Shrinkwrap version for ${pkg} (${shrinkwrapDep[pkg].version}) does not
          satisfy package.json version of ${packageDep[pkg]}.`);
            }
        }
    }
}
exports.ValidateShrinkwrapTask = ValidateShrinkwrapTask;
//# sourceMappingURL=ValidateShrinkwrapTask.js.map